'''OpenGL extension ARB.uniform_buffer_object

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_uniform_buffer_object'
_DEPRECATED = False
GL_UNIFORM_BUFFER = constant.Constant( 'GL_UNIFORM_BUFFER', 0x8A11 )
GL_UNIFORM_BUFFER_BINDING = constant.Constant( 'GL_UNIFORM_BUFFER_BINDING', 0x8A28 )
glget.addGLGetConstant( GL_UNIFORM_BUFFER_BINDING, (1,) )
GL_UNIFORM_BUFFER_START = constant.Constant( 'GL_UNIFORM_BUFFER_START', 0x8A29 )
GL_UNIFORM_BUFFER_SIZE = constant.Constant( 'GL_UNIFORM_BUFFER_SIZE', 0x8A2A )
GL_MAX_VERTEX_UNIFORM_BLOCKS = constant.Constant( 'GL_MAX_VERTEX_UNIFORM_BLOCKS', 0x8A2B )
glget.addGLGetConstant( GL_MAX_VERTEX_UNIFORM_BLOCKS, (1,) )
GL_MAX_GEOMETRY_UNIFORM_BLOCKS = constant.Constant( 'GL_MAX_GEOMETRY_UNIFORM_BLOCKS', 0x8A2C )
glget.addGLGetConstant( GL_MAX_GEOMETRY_UNIFORM_BLOCKS, (1,) )
GL_MAX_FRAGMENT_UNIFORM_BLOCKS = constant.Constant( 'GL_MAX_FRAGMENT_UNIFORM_BLOCKS', 0x8A2D )
glget.addGLGetConstant( GL_MAX_FRAGMENT_UNIFORM_BLOCKS, (1,) )
GL_MAX_COMBINED_UNIFORM_BLOCKS = constant.Constant( 'GL_MAX_COMBINED_UNIFORM_BLOCKS', 0x8A2E )
glget.addGLGetConstant( GL_MAX_COMBINED_UNIFORM_BLOCKS, (1,) )
GL_MAX_UNIFORM_BUFFER_BINDINGS = constant.Constant( 'GL_MAX_UNIFORM_BUFFER_BINDINGS', 0x8A2F )
glget.addGLGetConstant( GL_MAX_UNIFORM_BUFFER_BINDINGS, (1,) )
GL_MAX_UNIFORM_BLOCK_SIZE = constant.Constant( 'GL_MAX_UNIFORM_BLOCK_SIZE', 0x8A30 )
glget.addGLGetConstant( GL_MAX_UNIFORM_BLOCK_SIZE, (1,) )
GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS = constant.Constant( 'GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS', 0x8A31 )
glget.addGLGetConstant( GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS, (1,) )
GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS = constant.Constant( 'GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS', 0x8A32 )
glget.addGLGetConstant( GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS, (1,) )
GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS = constant.Constant( 'GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS', 0x8A33 )
glget.addGLGetConstant( GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS, (1,) )
GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT = constant.Constant( 'GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT', 0x8A34 )
glget.addGLGetConstant( GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT, (1,) )
GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH = constant.Constant( 'GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH', 0x8A35 )
GL_ACTIVE_UNIFORM_BLOCKS = constant.Constant( 'GL_ACTIVE_UNIFORM_BLOCKS', 0x8A36 )
GL_UNIFORM_TYPE = constant.Constant( 'GL_UNIFORM_TYPE', 0x8A37 )
GL_UNIFORM_SIZE = constant.Constant( 'GL_UNIFORM_SIZE', 0x8A38 )
GL_UNIFORM_NAME_LENGTH = constant.Constant( 'GL_UNIFORM_NAME_LENGTH', 0x8A39 )
GL_UNIFORM_BLOCK_INDEX = constant.Constant( 'GL_UNIFORM_BLOCK_INDEX', 0x8A3A )
GL_UNIFORM_OFFSET = constant.Constant( 'GL_UNIFORM_OFFSET', 0x8A3B )
GL_UNIFORM_ARRAY_STRIDE = constant.Constant( 'GL_UNIFORM_ARRAY_STRIDE', 0x8A3C )
GL_UNIFORM_MATRIX_STRIDE = constant.Constant( 'GL_UNIFORM_MATRIX_STRIDE', 0x8A3D )
GL_UNIFORM_IS_ROW_MAJOR = constant.Constant( 'GL_UNIFORM_IS_ROW_MAJOR', 0x8A3E )
GL_UNIFORM_BLOCK_BINDING = constant.Constant( 'GL_UNIFORM_BLOCK_BINDING', 0x8A3F )
GL_UNIFORM_BLOCK_DATA_SIZE = constant.Constant( 'GL_UNIFORM_BLOCK_DATA_SIZE', 0x8A40 )
GL_UNIFORM_BLOCK_NAME_LENGTH = constant.Constant( 'GL_UNIFORM_BLOCK_NAME_LENGTH', 0x8A41 )
GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS = constant.Constant( 'GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS', 0x8A42 )
GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES = constant.Constant( 'GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES', 0x8A43 )
GL_UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER = constant.Constant( 'GL_UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER', 0x8A44 )
GL_UNIFORM_BLOCK_REFERENCED_BY_GEOMETRY_SHADER = constant.Constant( 'GL_UNIFORM_BLOCK_REFERENCED_BY_GEOMETRY_SHADER', 0x8A45 )
GL_UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER = constant.Constant( 'GL_UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER', 0x8A46 )
GL_INVALID_INDEX = constant.Constant( 'GL_INVALID_INDEX', 0xFFFFFFFF )
glGetUniformIndices = platform.createExtensionFunction( 
'glGetUniformIndices',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,ctypes.POINTER( ctypes.POINTER( constants.GLchar )),arrays.GLuintArray,),
doc='glGetUniformIndices(GLuint(program), GLsizei(uniformCount), POINTER( ctypes.POINTER( constants.GLchar ))(uniformNames), GLuintArray(uniformIndices)) -> None',
argNames=('program','uniformCount','uniformNames','uniformIndices',),
deprecated=_DEPRECATED,
)

glGetActiveUniformsiv = platform.createExtensionFunction( 
'glGetActiveUniformsiv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLuintArray,constants.GLenum,arrays.GLintArray,),
doc='glGetActiveUniformsiv(GLuint(program), GLsizei(uniformCount), GLuintArray(uniformIndices), GLenum(pname), GLintArray(params)) -> None',
argNames=('program','uniformCount','uniformIndices','pname','params',),
deprecated=_DEPRECATED,
)

glGetActiveUniformName = platform.createExtensionFunction( 
'glGetActiveUniformName',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray,),
doc='glGetActiveUniformName(GLuint(program), GLuint(uniformIndex), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(uniformName)) -> None',
argNames=('program','uniformIndex','bufSize','length','uniformName',),
deprecated=_DEPRECATED,
)

glGetUniformBlockIndex = platform.createExtensionFunction( 
'glGetUniformBlockIndex',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLuint, 
argTypes=(constants.GLuint,arrays.GLcharArray,),
doc='glGetUniformBlockIndex(GLuint(program), GLcharArray(uniformBlockName)) -> constants.GLuint',
argNames=('program','uniformBlockName',),
deprecated=_DEPRECATED,
)

glGetActiveUniformBlockiv = platform.createExtensionFunction( 
'glGetActiveUniformBlockiv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetActiveUniformBlockiv(GLuint(program), GLuint(uniformBlockIndex), GLenum(pname), GLintArray(params)) -> None',
argNames=('program','uniformBlockIndex','pname','params',),
deprecated=_DEPRECATED,
)

glGetActiveUniformBlockName = platform.createExtensionFunction( 
'glGetActiveUniformBlockName',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray,),
doc='glGetActiveUniformBlockName(GLuint(program), GLuint(uniformBlockIndex), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(uniformBlockName)) -> None',
argNames=('program','uniformBlockIndex','bufSize','length','uniformBlockName',),
deprecated=_DEPRECATED,
)

glUniformBlockBinding = platform.createExtensionFunction( 
'glUniformBlockBinding',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLuint,),
doc='glUniformBlockBinding(GLuint(program), GLuint(uniformBlockIndex), GLuint(uniformBlockBinding)) -> None',
argNames=('program','uniformBlockIndex','uniformBlockBinding',),
deprecated=_DEPRECATED,
)


def glInitUniformBufferObjectARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.point_sprite

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_point_sprite'
_DEPRECATED = False
GL_POINT_SPRITE_NV = constant.Constant( 'GL_POINT_SPRITE_NV', 0x8861 )
glget.addGLGetConstant( GL_POINT_SPRITE_NV, (1,) )
GL_COORD_REPLACE_NV = constant.Constant( 'GL_COORD_REPLACE_NV', 0x8862 )
GL_POINT_SPRITE_R_MODE_NV = constant.Constant( 'GL_POINT_SPRITE_R_MODE_NV', 0x8863 )
glget.addGLGetConstant( GL_POINT_SPRITE_R_MODE_NV, (1,) )
glPointParameteriNV = platform.createExtensionFunction( 
'glPointParameteriNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,),
doc='glPointParameteriNV(GLenum(pname), GLint(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glPointParameterivNV = platform.createExtensionFunction( 
'glPointParameterivNV',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLintArray,),
doc='glPointParameterivNV(GLenum(pname), GLintArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)


def glInitPointSpriteNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIS.point_parameters

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIS_point_parameters'
_DEPRECATED = False
GL_POINT_SIZE_MIN_SGIS = constant.Constant( 'GL_POINT_SIZE_MIN_SGIS', 0x8126 )
GL_POINT_SIZE_MAX_SGIS = constant.Constant( 'GL_POINT_SIZE_MAX_SGIS', 0x8127 )
GL_POINT_FADE_THRESHOLD_SIZE_SGIS = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE_SGIS', 0x8128 )
GL_DISTANCE_ATTENUATION_SGIS = constant.Constant( 'GL_DISTANCE_ATTENUATION_SGIS', 0x8129 )
glPointParameterfSGIS = platform.createExtensionFunction( 
'glPointParameterfSGIS',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLfloat,),
doc='glPointParameterfSGIS(GLenum(pname), GLfloat(param)) -> None',
argNames=('pname','param',),
deprecated=_DEPRECATED,
)

glPointParameterfvSGIS = platform.createExtensionFunction( 
'glPointParameterfvSGIS',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLfloatArray,),
doc='glPointParameterfvSGIS(GLenum(pname), GLfloatArray(params)) -> None',
argNames=('pname','params',),
deprecated=_DEPRECATED,
)


def glInitPointParametersSGIS():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.multisample_filter_hint

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_multisample_filter_hint'
_DEPRECATED = False
GL_MULTISAMPLE_FILTER_HINT_NV = constant.Constant( 'GL_MULTISAMPLE_FILTER_HINT_NV', 0x8534 )
glget.addGLGetConstant( GL_MULTISAMPLE_FILTER_HINT_NV, (1,) )


def glInitMultisampleFilterHintNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.blend_func_separate

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_func_separate'
_DEPRECATED = False
GL_BLEND_DST_RGB_EXT = constant.Constant( 'GL_BLEND_DST_RGB_EXT', 0x80C8 )
glget.addGLGetConstant( GL_BLEND_DST_RGB_EXT, (1,) )
GL_BLEND_SRC_RGB_EXT = constant.Constant( 'GL_BLEND_SRC_RGB_EXT', 0x80C9 )
glget.addGLGetConstant( GL_BLEND_SRC_RGB_EXT, (1,) )
GL_BLEND_DST_ALPHA_EXT = constant.Constant( 'GL_BLEND_DST_ALPHA_EXT', 0x80CA )
glget.addGLGetConstant( GL_BLEND_DST_ALPHA_EXT, (1,) )
GL_BLEND_SRC_ALPHA_EXT = constant.Constant( 'GL_BLEND_SRC_ALPHA_EXT', 0x80CB )
glget.addGLGetConstant( GL_BLEND_SRC_ALPHA_EXT, (1,) )
glBlendFuncSeparateEXT = platform.createExtensionFunction( 
'glBlendFuncSeparateEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,constants.GLenum,),
doc='glBlendFuncSeparateEXT(GLenum(sfactorRGB), GLenum(dfactorRGB), GLenum(sfactorAlpha), GLenum(dfactorAlpha)) -> None',
argNames=('sfactorRGB','dfactorRGB','sfactorAlpha','dfactorAlpha',),
deprecated=_DEPRECATED,
)


def glInitBlendFuncSeparateEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.vertex_array_object

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.vertex_array_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines an interface that allows multiple sets of
	vertex array data to be cached in persistent server-side memory.
	It is intended to allow client data to be stored in memory that
	can be directly accessed by graphics hardware.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/vertex_array_object.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ATI.vertex_array_object import *
### END AUTOGENERATED SECTION


'''OpenGL extension ATI.vertex_attrib_array_object

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_vertex_attrib_array_object'
_DEPRECATED = False

glVertexAttribArrayObjectATI = platform.createExtensionFunction( 
'glVertexAttribArrayObjectATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,constants.GLenum,constants.GLboolean,constants.GLsizei,constants.GLuint,constants.GLuint,),
doc='glVertexAttribArrayObjectATI(GLuint(index), GLint(size), GLenum(type), GLboolean(normalized), GLsizei(stride), GLuint(buffer), GLuint(offset)) -> None',
argNames=('index','size','type','normalized','stride','buffer','offset',),
deprecated=_DEPRECATED,
)

glGetVertexAttribArrayObjectfvATI = platform.createExtensionFunction( 
'glGetVertexAttribArrayObjectfvATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLfloatArray,),
doc='glGetVertexAttribArrayObjectfvATI(GLuint(index), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)

glGetVertexAttribArrayObjectivATI = platform.createExtensionFunction( 
'glGetVertexAttribArrayObjectivATI',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetVertexAttribArrayObjectivATI(GLuint(index), GLenum(pname), GLintArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)


def glInitVertexAttribArrayObjectATI():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension VERSION.GL_1_5

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_1_5 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_1_5.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.VERSION.GL_1_5 import *
### END AUTOGENERATED SECTION
from OpenGL.lazywrapper import lazy
from OpenGL.arrays import ArrayDatatype

glDeleteBuffers = arrays.setInputArraySizeType(
    glDeleteBuffers,
    None,
    arrays.GLuintArray,
    'buffers',
)

glGenBuffers = wrapper.wrapper( glGenBuffers ).setOutput(
    'buffers', lambda n: (n,), 'n',
)

def _sizeOfArrayInput( pyArgs, index, wrapper ):
    return (
        arrays.ArrayDatatype.arrayByteCount( pyArgs[index] )
    )

@lazy( glBufferData )
def glBufferData( baseOperation, target, size, data=None, usage=None ):
    """Copy given data into the currently bound vertex-buffer-data object
    
    target -- the symbolic constant indicating which buffer type is intended
    size -- if provided, the count-in-bytes of the array
    data -- data-pointer to be used, may be None to initialize without 
        copying over a data-set 
    usage -- hint to the driver as to how to set up access to the buffer 
    
    Note: parameter "size" can be omitted, which makes the signature
        glBufferData( target, data, usage )
    instead of:
        glBufferData( target, size, data, usage )
    """
    if usage is None:
        usage = data 
        data = size 
        size = None 
    data = ArrayDatatype.asArray( data )
    if size is None:
        size = ArrayDatatype.arrayByteCount( data )
    return baseOperation( target, size, data, usage )

@lazy( glBufferSubData )
def glBufferSubData( baseOperation, target, offset, size, data=None ):
    """Copy subset of data into the currently bound vertex-buffer-data object
    
    target -- the symbolic constant indicating which buffer type is intended
    offset -- offset from beginning of buffer at which to copy bytes
    size -- the count-in-bytes of the array (if an int/long), if None,
        calculate size from data, if an array and data is None, use as 
        data (i.e. the parameter can be omitted and calculated)
    data -- data-pointer to be used, may be None to initialize without 
        copying over a data-set 
    
    Note that if size is not an int/long it is considered to be data
    """
    try:
        if size is not None:
            size = int( size )
    except TypeError, err:
        if data is not None:
            raise TypeError(
                """Expect an integer size *or* a data-array, not both"""
            )
        data = size 
        size = None 
    data = ArrayDatatype.asArray( data )
    if size is None:
        size = ArrayDatatype.arrayByteCount( data )
    return baseOperation( target, offset, size, data )

glGetBufferParameteriv = wrapper.wrapper(glGetBufferParameteriv).setOutput(
    "params",(1,),
)
@lazy( glGetBufferPointerv )
def glGetBufferPointerv( baseOperation, target, pname, params=None ):
    """Retrieve a ctypes pointer to buffer's data"""
    if params is None:
        size = glGetBufferParameteriv( target, GL_BUFFER_SIZE )
        data = arrays.ArrayDatatype.zeros( (size,), GL_UNSIGNED_BYTE )
        result = baseOperation( target, pname, ctypes.byref( data ) )
        return data
    else:
        return baseOperation( target, pname, params )

@lazy( glDeleteQueries )
def glDeleteQueries( baseOperation, n, ids=None ):
    if ids is None:
        ids = arrays.GLuintArray.asArray( ids )
        n = arrays.GLuintArray.arraySize( ids )
    else:
        ids = arrays.GLuintArray.asArray( ids )
    return baseOperation( n,ids )
@lazy( glGenQueries )
def glGenQueries( baseOperation, n, ids=None ):
    """Generate n queries, if ids is None, is allocated
    
    returns array of ids
    """
    if ids is None:
        ids = arrays.GLuintArray.zeros( (n,))
    else:
        ids = arrays.GLuintArray.asArray( ids )
    baseOperation( n, ids )
    return ids

for func in (
    'glGetQueryiv','glGetQueryObjectiv','glGetQueryObjectuiv',
):
    globals()[func] = wrapper.wrapper(globals()[func]).setOutput(
        "params", (1,)
    )
del func, glget



'''OpenGL extension ARB.provoking_vertex

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_provoking_vertex'
_DEPRECATED = False
GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION = constant.Constant( 'GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION', 0x8E4C )
glget.addGLGetConstant( GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION, (1,) )
GL_FIRST_VERTEX_CONVENTION = constant.Constant( 'GL_FIRST_VERTEX_CONVENTION', 0x8E4D )
GL_LAST_VERTEX_CONVENTION = constant.Constant( 'GL_LAST_VERTEX_CONVENTION', 0x8E4E )
GL_PROVOKING_VERTEX = constant.Constant( 'GL_PROVOKING_VERTEX', 0x8E4F )
glget.addGLGetConstant( GL_PROVOKING_VERTEX, (1,) )
glProvokingVertex = platform.createExtensionFunction( 
'glProvokingVertex',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glProvokingVertex(GLenum(mode)) -> None',
argNames=('mode',),
deprecated=_DEPRECATED,
)


def glInitProvokingVertexARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.framebuffer_blit

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_framebuffer_blit'
_DEPRECATED = False
GL_READ_FRAMEBUFFER_EXT = constant.Constant( 'GL_READ_FRAMEBUFFER_EXT', 0x8CA8 )
GL_DRAW_FRAMEBUFFER_EXT = constant.Constant( 'GL_DRAW_FRAMEBUFFER_EXT', 0x8CA9 )
GL_READ_FRAMEBUFFER_BINDING_EXT = constant.Constant( 'GL_READ_FRAMEBUFFER_BINDING_EXT', 0x8CAA )
glBlitFramebufferEXT = platform.createExtensionFunction( 
'glBlitFramebufferEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,constants.GLbitfield,constants.GLenum,),
doc='glBlitFramebufferEXT(GLint(srcX0), GLint(srcY0), GLint(srcX1), GLint(srcY1), GLint(dstX0), GLint(dstY0), GLint(dstX1), GLint(dstY1), GLbitfield(mask), GLenum(filter)) -> None',
argNames=('srcX0','srcY0','srcX1','srcY1','dstX0','dstY0','dstX1','dstY1','mask','filter',),
deprecated=_DEPRECATED,
)


def glInitFramebufferBlitEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.vertex_program4

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_vertex_program4'
_DEPRECATED = False
GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV = constant.Constant( 'GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV', 0x88FD )
glVertexAttribI1iEXT = platform.createExtensionFunction( 
'glVertexAttribI1iEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,),
doc='glVertexAttribI1iEXT(GLuint(index), GLint(x)) -> None',
argNames=('index','x',),
deprecated=_DEPRECATED,
)

glVertexAttribI2iEXT = platform.createExtensionFunction( 
'glVertexAttribI2iEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,constants.GLint,),
doc='glVertexAttribI2iEXT(GLuint(index), GLint(x), GLint(y)) -> None',
argNames=('index','x','y',),
deprecated=_DEPRECATED,
)

glVertexAttribI3iEXT = platform.createExtensionFunction( 
'glVertexAttribI3iEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,constants.GLint,constants.GLint,),
doc='glVertexAttribI3iEXT(GLuint(index), GLint(x), GLint(y), GLint(z)) -> None',
argNames=('index','x','y','z',),
deprecated=_DEPRECATED,
)

glVertexAttribI4iEXT = platform.createExtensionFunction( 
'glVertexAttribI4iEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,constants.GLint,constants.GLint,constants.GLint,),
doc='glVertexAttribI4iEXT(GLuint(index), GLint(x), GLint(y), GLint(z), GLint(w)) -> None',
argNames=('index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glVertexAttribI1uiEXT = platform.createExtensionFunction( 
'glVertexAttribI1uiEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,),
doc='glVertexAttribI1uiEXT(GLuint(index), GLuint(x)) -> None',
argNames=('index','x',),
deprecated=_DEPRECATED,
)

glVertexAttribI2uiEXT = platform.createExtensionFunction( 
'glVertexAttribI2uiEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLuint,),
doc='glVertexAttribI2uiEXT(GLuint(index), GLuint(x), GLuint(y)) -> None',
argNames=('index','x','y',),
deprecated=_DEPRECATED,
)

glVertexAttribI3uiEXT = platform.createExtensionFunction( 
'glVertexAttribI3uiEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLuint,constants.GLuint,),
doc='glVertexAttribI3uiEXT(GLuint(index), GLuint(x), GLuint(y), GLuint(z)) -> None',
argNames=('index','x','y','z',),
deprecated=_DEPRECATED,
)

glVertexAttribI4uiEXT = platform.createExtensionFunction( 
'glVertexAttribI4uiEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLuint,constants.GLuint,constants.GLuint,),
doc='glVertexAttribI4uiEXT(GLuint(index), GLuint(x), GLuint(y), GLuint(z), GLuint(w)) -> None',
argNames=('index','x','y','z','w',),
deprecated=_DEPRECATED,
)

glVertexAttribI1ivEXT = platform.createExtensionFunction( 
'glVertexAttribI1ivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,),
doc='glVertexAttribI1ivEXT(GLuint(index), GLintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI2ivEXT = platform.createExtensionFunction( 
'glVertexAttribI2ivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,),
doc='glVertexAttribI2ivEXT(GLuint(index), GLintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI3ivEXT = platform.createExtensionFunction( 
'glVertexAttribI3ivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,),
doc='glVertexAttribI3ivEXT(GLuint(index), GLintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI4ivEXT = platform.createExtensionFunction( 
'glVertexAttribI4ivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,),
doc='glVertexAttribI4ivEXT(GLuint(index), GLintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI1uivEXT = platform.createExtensionFunction( 
'glVertexAttribI1uivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLuintArray,),
doc='glVertexAttribI1uivEXT(GLuint(index), GLuintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI2uivEXT = platform.createExtensionFunction( 
'glVertexAttribI2uivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLuintArray,),
doc='glVertexAttribI2uivEXT(GLuint(index), GLuintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI3uivEXT = platform.createExtensionFunction( 
'glVertexAttribI3uivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLuintArray,),
doc='glVertexAttribI3uivEXT(GLuint(index), GLuintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI4uivEXT = platform.createExtensionFunction( 
'glVertexAttribI4uivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLuintArray,),
doc='glVertexAttribI4uivEXT(GLuint(index), GLuintArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI4bvEXT = platform.createExtensionFunction( 
'glVertexAttribI4bvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLbyteArray,),
doc='glVertexAttribI4bvEXT(GLuint(index), GLbyteArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI4svEXT = platform.createExtensionFunction( 
'glVertexAttribI4svEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLshortArray,),
doc='glVertexAttribI4svEXT(GLuint(index), GLshortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI4ubvEXT = platform.createExtensionFunction( 
'glVertexAttribI4ubvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLubyteArray,),
doc='glVertexAttribI4ubvEXT(GLuint(index), GLubyteArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribI4usvEXT = platform.createExtensionFunction( 
'glVertexAttribI4usvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLushortArray,),
doc='glVertexAttribI4usvEXT(GLuint(index), GLushortArray(v)) -> None',
argNames=('index','v',),
deprecated=_DEPRECATED,
)

glVertexAttribIPointerEXT = platform.createExtensionFunction( 
'glVertexAttribIPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLint,constants.GLenum,constants.GLsizei,ctypes.c_void_p,),
doc='glVertexAttribIPointerEXT(GLuint(index), GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer)) -> None',
argNames=('index','size','type','stride','pointer',),
deprecated=_DEPRECATED,
)

glGetVertexAttribIivEXT = platform.createExtensionFunction( 
'glGetVertexAttribIivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetVertexAttribIivEXT(GLuint(index), GLenum(pname), GLintArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)

glGetVertexAttribIuivEXT = platform.createExtensionFunction( 
'glGetVertexAttribIuivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,arrays.GLuintArray,),
doc='glGetVertexAttribIuivEXT(GLuint(index), GLenum(pname), GLuintArray(params)) -> None',
argNames=('index','pname','params',),
deprecated=_DEPRECATED,
)


def glInitVertexProgram4NV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""OpenGL version 1.3 imaging-handling routines"""
from OpenGL import wrapper, constants, arrays
from OpenGL.raw.GL.VERSION import GL_1_3 as simple
from OpenGL.GL import images, glget

for dimensions in (1,2,3):
    for function in ('glCompressedTexImage%sD','glCompressedTexSubImage%sD'):
        name = function%(dimensions,)
        globals()[ name ] = images.compressedImageFunction(
            getattr( simple, name )
        )
        del name, function
    del dimensions

if simple.glGetCompressedTexImage:
    def glGetCompressedTexImage( target, level, img=None ):
        """Retrieve a compressed texture image"""
        if img is None:
            length = glget.glGetTexLevelParameteriv(
                target, 0, 
                simple.GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB, 
            )
            img = arrays.ArrayDataType.zeros( (length,), constants.GL_UNSIGNED_BYTE )
        return simple.glGetCompressedTexImage(target, 0, img);


'''OpenGL extension EXT.shared_texture_palette

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_shared_texture_palette'
_DEPRECATED = False
GL_SHARED_TEXTURE_PALETTE_EXT = constant.Constant( 'GL_SHARED_TEXTURE_PALETTE_EXT', 0x81FB )


def glInitSharedTexturePaletteEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )




if __name__ == "__main__":
    from OpenGL.GLUT import *
    from OpenGL.GL import *
    glutInit( ' ' )
    glutInitDisplayMode( GLUT_SINGLE )
    window = glutCreateWindow("hello")
    glutDisplayFunc( lambda *args: 1 )
    #glutMainLoop()



'''OpenGL extension INTEL.parallel_arrays

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_INTEL_parallel_arrays'
_DEPRECATED = False
GL_PARALLEL_ARRAYS_INTEL = constant.Constant( 'GL_PARALLEL_ARRAYS_INTEL', 0x83F4 )
glget.addGLGetConstant( GL_PARALLEL_ARRAYS_INTEL, (1,) )
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F5 )
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F6 )
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F7 )
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F8 )
glVertexPointervINTEL = platform.createExtensionFunction( 
'glVertexPointervINTEL',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glVertexPointervINTEL(GLint(size), GLenum(type), POINTER(ctypes.c_void_p)(pointer)) -> None',
argNames=('size','type','pointer',),
deprecated=_DEPRECATED,
)

glNormalPointervINTEL = platform.createExtensionFunction( 
'glNormalPointervINTEL',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glNormalPointervINTEL(GLenum(type), POINTER(ctypes.c_void_p)(pointer)) -> None',
argNames=('type','pointer',),
deprecated=_DEPRECATED,
)

glColorPointervINTEL = platform.createExtensionFunction( 
'glColorPointervINTEL',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glColorPointervINTEL(GLint(size), GLenum(type), POINTER(ctypes.c_void_p)(pointer)) -> None',
argNames=('size','type','pointer',),
deprecated=_DEPRECATED,
)

glTexCoordPointervINTEL = platform.createExtensionFunction( 
'glTexCoordPointervINTEL',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glTexCoordPointervINTEL(GLint(size), GLenum(type), POINTER(ctypes.c_void_p)(pointer)) -> None',
argNames=('size','type','pointer',),
deprecated=_DEPRECATED,
)


def glInitParallelArraysINTEL():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



import pygame
from pygame.locals import *
import pygame.key
import pygame.display
from OpenGL.GLUT import *


def createPyGameContext():
    """Setup a minimal PyGame context for testing"""
    pygame.display.init()
    screen = pygame.display.set_mode(
        (300,300),
        OPENGL | DOUBLEBUF,
    )
    pygame.display.set_caption("testing")
    pygame.key.set_repeat(500,30)
    return pygame.display

def createGLUTContext():
    """Setup a minimal GLUT context for testing"""
    glutInit( [] )
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB )
    glutInitWindowSize(250, 250)
    glutInitWindowPosition(100, 100)
    window = glutCreateWindow("hello")
    return window


'''OpenGL extension NV.light_max_exponent

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_light_max_exponent'
_DEPRECATED = False
GL_MAX_SHININESS_NV = constant.Constant( 'GL_MAX_SHININESS_NV', 0x8504 )
glget.addGLGetConstant( GL_MAX_SHININESS_NV, (1,) )
GL_MAX_SPOT_EXPONENT_NV = constant.Constant( 'GL_MAX_SPOT_EXPONENT_NV', 0x8505 )
glget.addGLGetConstant( GL_MAX_SPOT_EXPONENT_NV, (1,) )


def glInitLightMaxExponentNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.imaging

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_imaging'
_DEPRECATED = False
GL_CONSTANT_COLOR = constant.Constant( 'GL_CONSTANT_COLOR', 0x8001 )
GL_ONE_MINUS_CONSTANT_COLOR = constant.Constant( 'GL_ONE_MINUS_CONSTANT_COLOR', 0x8002 )
GL_CONSTANT_ALPHA = constant.Constant( 'GL_CONSTANT_ALPHA', 0x8003 )
GL_ONE_MINUS_CONSTANT_ALPHA = constant.Constant( 'GL_ONE_MINUS_CONSTANT_ALPHA', 0x8004 )
GL_BLEND_COLOR = constant.Constant( 'GL_BLEND_COLOR', 0x8005 )
GL_FUNC_ADD = constant.Constant( 'GL_FUNC_ADD', 0x8006 )
GL_MIN = constant.Constant( 'GL_MIN', 0x8007 )
GL_MAX = constant.Constant( 'GL_MAX', 0x8008 )
GL_BLEND_EQUATION = constant.Constant( 'GL_BLEND_EQUATION', 0x8009 )
GL_FUNC_SUBTRACT = constant.Constant( 'GL_FUNC_SUBTRACT', 0x800A )
GL_FUNC_REVERSE_SUBTRACT = constant.Constant( 'GL_FUNC_REVERSE_SUBTRACT', 0x800B )
# import legacy entry points to allow checking for bool(entryPoint)
from OpenGL.raw.GL.ARB.imaging_DEPRECATED import *

def glInitImagingARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.clipmap

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.clipmap to provide a more 
Python-friendly API

Overview (from the spec)
	
	Mipmaps provide a general but expensive solution when the texture image
	is very large.  This extension defines clipmaps, which occupy a small
	subset of the memory required by equivalent mipmaps, but provide much
	of the mipmap rendering capabilities.  Clipmaps are especially useful
	for rendering terrain.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/clipmap.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.SGIX.clipmap import *
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.blend_subtract

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_subtract'
_DEPRECATED = False
GL_FUNC_SUBTRACT_EXT = constant.Constant( 'GL_FUNC_SUBTRACT_EXT', 0x800A )
GL_FUNC_REVERSE_SUBTRACT_EXT = constant.Constant( 'GL_FUNC_REVERSE_SUBTRACT_EXT', 0x800B )


def glInitBlendSubtractEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.color_matrix

The official definition of this extension is available here:
    http://oss.sgi.com/projects/ogl-sample/registry/EXT/color_matrix.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes



def glInitColorMatrixEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( 'GL_EXT_color_matrix' )


'''OpenGL extension ARB.half_float_vertex

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_half_float_vertex'
_DEPRECATED = False
GL_HALF_FLOAT = constant.Constant( 'GL_HALF_FLOAT', 0x140B )


def glInitHalfFloatVertexARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension PGI.vertex_hints

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_PGI_vertex_hints'
_DEPRECATED = False
GL_VERTEX_DATA_HINT_PGI = constant.Constant( 'GL_VERTEX_DATA_HINT_PGI', 0x1A22A )
GL_VERTEX_CONSISTENT_HINT_PGI = constant.Constant( 'GL_VERTEX_CONSISTENT_HINT_PGI', 0x1A22B )
GL_MATERIAL_SIDE_HINT_PGI = constant.Constant( 'GL_MATERIAL_SIDE_HINT_PGI', 0x1A22C )
GL_MAX_VERTEX_HINT_PGI = constant.Constant( 'GL_MAX_VERTEX_HINT_PGI', 0x1A22D )
GL_COLOR3_BIT_PGI = constant.Constant( 'GL_COLOR3_BIT_PGI', 0x10000 )
GL_COLOR4_BIT_PGI = constant.Constant( 'GL_COLOR4_BIT_PGI', 0x20000 )
GL_EDGEFLAG_BIT_PGI = constant.Constant( 'GL_EDGEFLAG_BIT_PGI', 0x40000 )
GL_INDEX_BIT_PGI = constant.Constant( 'GL_INDEX_BIT_PGI', 0x80000 )
GL_MAT_AMBIENT_BIT_PGI = constant.Constant( 'GL_MAT_AMBIENT_BIT_PGI', 0x100000 )
GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI = constant.Constant( 'GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI', 0x200000 )
GL_MAT_DIFFUSE_BIT_PGI = constant.Constant( 'GL_MAT_DIFFUSE_BIT_PGI', 0x400000 )
GL_MAT_EMISSION_BIT_PGI = constant.Constant( 'GL_MAT_EMISSION_BIT_PGI', 0x800000 )
GL_MAT_COLOR_INDEXES_BIT_PGI = constant.Constant( 'GL_MAT_COLOR_INDEXES_BIT_PGI', 0x1000000 )
GL_MAT_SHININESS_BIT_PGI = constant.Constant( 'GL_MAT_SHININESS_BIT_PGI', 0x2000000 )
GL_MAT_SPECULAR_BIT_PGI = constant.Constant( 'GL_MAT_SPECULAR_BIT_PGI', 0x4000000 )
GL_NORMAL_BIT_PGI = constant.Constant( 'GL_NORMAL_BIT_PGI', 0x8000000 )
GL_TEXCOORD1_BIT_PGI = constant.Constant( 'GL_TEXCOORD1_BIT_PGI', 0x10000000 )
GL_TEXCOORD2_BIT_PGI = constant.Constant( 'GL_TEXCOORD2_BIT_PGI', 0x20000000 )
GL_TEXCOORD3_BIT_PGI = constant.Constant( 'GL_TEXCOORD3_BIT_PGI', 0x40000000 )
GL_TEXCOORD4_BIT_PGI = constant.Constant( 'GL_TEXCOORD4_BIT_PGI', 0x80000000 )
GL_VERTEX23_BIT_PGI = constant.Constant( 'GL_VERTEX23_BIT_PGI', 0x4 )
GL_VERTEX4_BIT_PGI = constant.Constant( 'GL_VERTEX4_BIT_PGI', 0x8 )


def glInitVertexHintsPGI():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



from OpenGL.GL import *
from OpenGL.GLUT import *
import sys
import OpenGL.GL.EXT.framebuffer_object as EXT
import OpenGL.GL.ARB.framebuffer_object as ARB

def main():
    glutInit(sys.argv)
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH)	
    glutInitWindowSize(640, 480)	
    glutInitWindowPosition(0, 0)	
    window = glutCreateWindow("Framebuffer bug demo")

    for i in range( 200 ):
        fbo = EXT.glGenFramebuffersEXT(1)
        print "FBO = ", fbo
        EXT.glDeleteFramebuffersEXT (int(fbo))
        fbo = ARB.glGenFramebuffers(1)
        print "FBO = ", fbo
        ARB.glDeleteFramebuffers(int(fbo))
        glFlush()

# Print message to console, and kick off the main to get it rolling.
main()
        



"""Run-time calculation of offset into Python Numeric (old) structures

Numeric Python, by fortuitous chance, puts the one thing
we need precisely as the first value in the structure beyond the
PyObject * header, so that it's exactly that many bytes from the
pointer value for the object...
"""
import ctypes

def dataPointerFunction( ):
    """Calculate the data-pointer offset in the Numeric object header"""
    offset = object.__basicsize__
    from_address = ctypes.c_void_p.from_address
    def dataPointer( data):
        """Return pointer-to-data + offset"""
        return from_address( id( data ) + offset ).value
    return dataPointer

dataPointer = dataPointerFunction()

if __name__ == "__main__":
    import Numeric 
    test = Numeric.arange( 0,200, 1,'i' )
    aType = ctypes.c_int * 200
    test2 = aType.from_address( dataPointer( test ) )
    assert test == test2, (test,test2)


'''OpenGL extension MESA.ycbcr_texture

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_MESA_ycbcr_texture'
_DEPRECATED = False
GL_UNSIGNED_SHORT_8_8_MESA = constant.Constant( 'GL_UNSIGNED_SHORT_8_8_MESA', 0x85BA )
GL_UNSIGNED_SHORT_8_8_REV_MESA = constant.Constant( 'GL_UNSIGNED_SHORT_8_8_REV_MESA', 0x85BB )
GL_YCBCR_MESA = constant.Constant( 'GL_YCBCR_MESA', 0x8757 )


def glInitYcbcrTextureMESA():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.fragment_program

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_fragment_program'
_DEPRECATED = False
GL_FRAGMENT_PROGRAM_ARB = constant.Constant( 'GL_FRAGMENT_PROGRAM_ARB', 0x8804 )
glget.addGLGetConstant( GL_FRAGMENT_PROGRAM_ARB, (1,) )
GL_PROGRAM_ALU_INSTRUCTIONS_ARB = constant.Constant( 'GL_PROGRAM_ALU_INSTRUCTIONS_ARB', 0x8805 )
GL_PROGRAM_TEX_INSTRUCTIONS_ARB = constant.Constant( 'GL_PROGRAM_TEX_INSTRUCTIONS_ARB', 0x8806 )
GL_PROGRAM_TEX_INDIRECTIONS_ARB = constant.Constant( 'GL_PROGRAM_TEX_INDIRECTIONS_ARB', 0x8807 )
GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB', 0x8808 )
GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB', 0x8809 )
GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = constant.Constant( 'GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB', 0x880A )
GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB', 0x880B )
GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB', 0x880C )
GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB', 0x880D )
GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB', 0x880E )
GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB', 0x880F )
GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = constant.Constant( 'GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB', 0x8810 )
GL_MAX_TEXTURE_COORDS_ARB = constant.Constant( 'GL_MAX_TEXTURE_COORDS_ARB', 0x8871 )
glget.addGLGetConstant( GL_MAX_TEXTURE_COORDS_ARB, (1,) )
GL_MAX_TEXTURE_IMAGE_UNITS_ARB = constant.Constant( 'GL_MAX_TEXTURE_IMAGE_UNITS_ARB', 0x8872 )
glget.addGLGetConstant( GL_MAX_TEXTURE_IMAGE_UNITS_ARB, (1,) )


def glInitFragmentProgramARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.copy_depth_to_color

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_copy_depth_to_color'
_DEPRECATED = False
GL_DEPTH_STENCIL_TO_RGBA_NV = constant.Constant( 'GL_DEPTH_STENCIL_TO_RGBA_NV', 0x886E )
GL_DEPTH_STENCIL_TO_BGRA_NV = constant.Constant( 'GL_DEPTH_STENCIL_TO_BGRA_NV', 0x886F )


def glInitCopyDepthToColorNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension S3.s3tc

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_S3_s3tc'
_DEPRECATED = False
GL_RGB_S3TC = constant.Constant( 'GL_RGB_S3TC', 0x83A0 )
GL_RGB4_S3TC = constant.Constant( 'GL_RGB4_S3TC', 0x83A1 )
GL_RGBA_S3TC = constant.Constant( 'GL_RGBA_S3TC', 0x83A2 )
GL_RGBA4_S3TC = constant.Constant( 'GL_RGBA4_S3TC', 0x83A3 )


def glInitS3TcS3():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SUNX.constant_data

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SUNX_constant_data'
_DEPRECATED = False
GL_UNPACK_CONSTANT_DATA_SUNX = constant.Constant( 'GL_UNPACK_CONSTANT_DATA_SUNX', 0x81D5 )
GL_TEXTURE_CONSTANT_DATA_SUNX = constant.Constant( 'GL_TEXTURE_CONSTANT_DATA_SUNX', 0x81D6 )
glFinishTextureSUNX = platform.createExtensionFunction( 
'glFinishTextureSUNX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(),
doc='glFinishTextureSUNX() -> None',
argNames=(),
deprecated=_DEPRECATED,
)


def glInitConstantDataSUNX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.texture_env_combine

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_texture_env_combine'
_DEPRECATED = False
GL_COMBINE_ARB = constant.Constant( 'GL_COMBINE_ARB', 0x8570 )
GL_COMBINE_RGB_ARB = constant.Constant( 'GL_COMBINE_RGB_ARB', 0x8571 )
GL_COMBINE_ALPHA_ARB = constant.Constant( 'GL_COMBINE_ALPHA_ARB', 0x8572 )
GL_SOURCE0_RGB_ARB = constant.Constant( 'GL_SOURCE0_RGB_ARB', 0x8580 )
GL_SOURCE1_RGB_ARB = constant.Constant( 'GL_SOURCE1_RGB_ARB', 0x8581 )
GL_SOURCE2_RGB_ARB = constant.Constant( 'GL_SOURCE2_RGB_ARB', 0x8582 )
GL_SOURCE0_ALPHA_ARB = constant.Constant( 'GL_SOURCE0_ALPHA_ARB', 0x8588 )
GL_SOURCE1_ALPHA_ARB = constant.Constant( 'GL_SOURCE1_ALPHA_ARB', 0x8589 )
GL_SOURCE2_ALPHA_ARB = constant.Constant( 'GL_SOURCE2_ALPHA_ARB', 0x858A )
GL_OPERAND0_RGB_ARB = constant.Constant( 'GL_OPERAND0_RGB_ARB', 0x8590 )
GL_OPERAND1_RGB_ARB = constant.Constant( 'GL_OPERAND1_RGB_ARB', 0x8591 )
GL_OPERAND2_RGB_ARB = constant.Constant( 'GL_OPERAND2_RGB_ARB', 0x8592 )
GL_OPERAND0_ALPHA_ARB = constant.Constant( 'GL_OPERAND0_ALPHA_ARB', 0x8598 )
GL_OPERAND1_ALPHA_ARB = constant.Constant( 'GL_OPERAND1_ALPHA_ARB', 0x8599 )
GL_OPERAND2_ALPHA_ARB = constant.Constant( 'GL_OPERAND2_ALPHA_ARB', 0x859A )
GL_RGB_SCALE_ARB = constant.Constant( 'GL_RGB_SCALE_ARB', 0x8573 )
GL_ADD_SIGNED_ARB = constant.Constant( 'GL_ADD_SIGNED_ARB', 0x8574 )
GL_INTERPOLATE_ARB = constant.Constant( 'GL_INTERPOLATE_ARB', 0x8575 )
GL_SUBTRACT_ARB = constant.Constant( 'GL_SUBTRACT_ARB', 0x84E7 )
GL_CONSTANT_ARB = constant.Constant( 'GL_CONSTANT_ARB', 0x8576 )
GL_PRIMARY_COLOR_ARB = constant.Constant( 'GL_PRIMARY_COLOR_ARB', 0x8577 )
GL_PREVIOUS_ARB = constant.Constant( 'GL_PREVIOUS_ARB', 0x8578 )


def glInitTextureEnvCombineARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shading_language_100

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shading_language_100 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension string indicates that the OpenGL Shading Language is
	supported. The Shading Language is defined by a separate specification
	document which can be downloaded from
	
	    http://www.opengl.org/documentation/oglsl.html

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shading_language_100.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.shading_language_100 import *
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.convolution

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_convolution'
_DEPRECATED = False
GL_CONVOLUTION_1D_EXT = constant.Constant( 'GL_CONVOLUTION_1D_EXT', 0x8010 )
glget.addGLGetConstant( GL_CONVOLUTION_1D_EXT, (1,) )
GL_CONVOLUTION_2D_EXT = constant.Constant( 'GL_CONVOLUTION_2D_EXT', 0x8011 )
glget.addGLGetConstant( GL_CONVOLUTION_2D_EXT, (1,) )
GL_SEPARABLE_2D_EXT = constant.Constant( 'GL_SEPARABLE_2D_EXT', 0x8012 )
glget.addGLGetConstant( GL_SEPARABLE_2D_EXT, (1,) )
GL_CONVOLUTION_BORDER_MODE_EXT = constant.Constant( 'GL_CONVOLUTION_BORDER_MODE_EXT', 0x8013 )
GL_CONVOLUTION_FILTER_SCALE_EXT = constant.Constant( 'GL_CONVOLUTION_FILTER_SCALE_EXT', 0x8014 )
GL_CONVOLUTION_FILTER_BIAS_EXT = constant.Constant( 'GL_CONVOLUTION_FILTER_BIAS_EXT', 0x8015 )
GL_REDUCE_EXT = constant.Constant( 'GL_REDUCE_EXT', 0x8016 )
GL_CONVOLUTION_FORMAT_EXT = constant.Constant( 'GL_CONVOLUTION_FORMAT_EXT', 0x8017 )
GL_CONVOLUTION_WIDTH_EXT = constant.Constant( 'GL_CONVOLUTION_WIDTH_EXT', 0x8018 )
GL_CONVOLUTION_HEIGHT_EXT = constant.Constant( 'GL_CONVOLUTION_HEIGHT_EXT', 0x8019 )
GL_MAX_CONVOLUTION_WIDTH_EXT = constant.Constant( 'GL_MAX_CONVOLUTION_WIDTH_EXT', 0x801A )
GL_MAX_CONVOLUTION_HEIGHT_EXT = constant.Constant( 'GL_MAX_CONVOLUTION_HEIGHT_EXT', 0x801B )
GL_POST_CONVOLUTION_RED_SCALE_EXT = constant.Constant( 'GL_POST_CONVOLUTION_RED_SCALE_EXT', 0x801C )
glget.addGLGetConstant( GL_POST_CONVOLUTION_RED_SCALE_EXT, (1,) )
GL_POST_CONVOLUTION_GREEN_SCALE_EXT = constant.Constant( 'GL_POST_CONVOLUTION_GREEN_SCALE_EXT', 0x801D )
glget.addGLGetConstant( GL_POST_CONVOLUTION_GREEN_SCALE_EXT, (1,) )
GL_POST_CONVOLUTION_BLUE_SCALE_EXT = constant.Constant( 'GL_POST_CONVOLUTION_BLUE_SCALE_EXT', 0x801E )
glget.addGLGetConstant( GL_POST_CONVOLUTION_BLUE_SCALE_EXT, (1,) )
GL_POST_CONVOLUTION_ALPHA_SCALE_EXT = constant.Constant( 'GL_POST_CONVOLUTION_ALPHA_SCALE_EXT', 0x801F )
glget.addGLGetConstant( GL_POST_CONVOLUTION_ALPHA_SCALE_EXT, (1,) )
GL_POST_CONVOLUTION_RED_BIAS_EXT = constant.Constant( 'GL_POST_CONVOLUTION_RED_BIAS_EXT', 0x8020 )
glget.addGLGetConstant( GL_POST_CONVOLUTION_RED_BIAS_EXT, (1,) )
GL_POST_CONVOLUTION_GREEN_BIAS_EXT = constant.Constant( 'GL_POST_CONVOLUTION_GREEN_BIAS_EXT', 0x8021 )
glget.addGLGetConstant( GL_POST_CONVOLUTION_GREEN_BIAS_EXT, (1,) )
GL_POST_CONVOLUTION_BLUE_BIAS_EXT = constant.Constant( 'GL_POST_CONVOLUTION_BLUE_BIAS_EXT', 0x8022 )
glget.addGLGetConstant( GL_POST_CONVOLUTION_BLUE_BIAS_EXT, (1,) )
GL_POST_CONVOLUTION_ALPHA_BIAS_EXT = constant.Constant( 'GL_POST_CONVOLUTION_ALPHA_BIAS_EXT', 0x8023 )
glget.addGLGetConstant( GL_POST_CONVOLUTION_ALPHA_BIAS_EXT, (1,) )
glConvolutionFilter1DEXT = platform.createExtensionFunction( 
'glConvolutionFilter1DEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glConvolutionFilter1DEXT(GLenum(target), GLenum(internalformat), GLsizei(width), GLenum(format), GLenum(type), c_void_p(image)) -> None',
argNames=('target','internalformat','width','format','type','image',),
deprecated=_DEPRECATED,
)

glConvolutionFilter2DEXT = platform.createExtensionFunction( 
'glConvolutionFilter2DEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glConvolutionFilter2DEXT(GLenum(target), GLenum(internalformat), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), c_void_p(image)) -> None',
argNames=('target','internalformat','width','height','format','type','image',),
deprecated=_DEPRECATED,
)

glConvolutionParameterfEXT = platform.createExtensionFunction( 
'glConvolutionParameterfEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLfloat,),
doc='glConvolutionParameterfEXT(GLenum(target), GLenum(pname), GLfloat(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glConvolutionParameterfvEXT = platform.createExtensionFunction( 
'glConvolutionParameterfvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glConvolutionParameterfvEXT(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glConvolutionParameteriEXT = platform.createExtensionFunction( 
'glConvolutionParameteriEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,),
doc='glConvolutionParameteriEXT(GLenum(target), GLenum(pname), GLint(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glConvolutionParameterivEXT = platform.createExtensionFunction( 
'glConvolutionParameterivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glConvolutionParameterivEXT(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glCopyConvolutionFilter1DEXT = platform.createExtensionFunction( 
'glCopyConvolutionFilter1DEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,constants.GLint,constants.GLsizei,),
doc='glCopyConvolutionFilter1DEXT(GLenum(target), GLenum(internalformat), GLint(x), GLint(y), GLsizei(width)) -> None',
argNames=('target','internalformat','x','y','width',),
deprecated=_DEPRECATED,
)

glCopyConvolutionFilter2DEXT = platform.createExtensionFunction( 
'glCopyConvolutionFilter2DEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,constants.GLint,constants.GLsizei,constants.GLsizei,),
doc='glCopyConvolutionFilter2DEXT(GLenum(target), GLenum(internalformat), GLint(x), GLint(y), GLsizei(width), GLsizei(height)) -> None',
argNames=('target','internalformat','x','y','width','height',),
deprecated=_DEPRECATED,
)

glGetConvolutionFilterEXT = platform.createExtensionFunction( 
'glGetConvolutionFilterEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,ctypes.c_void_p,),
doc='glGetConvolutionFilterEXT(GLenum(target), GLenum(format), GLenum(type), c_void_p(image)) -> None',
argNames=('target','format','type','image',),
deprecated=_DEPRECATED,
)

glGetConvolutionParameterfvEXT = platform.createExtensionFunction( 
'glGetConvolutionParameterfvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLfloatArray,),
doc='glGetConvolutionParameterfvEXT(GLenum(target), GLenum(pname), GLfloatArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetConvolutionParameterivEXT = platform.createExtensionFunction( 
'glGetConvolutionParameterivEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,arrays.GLintArray,),
doc='glGetConvolutionParameterivEXT(GLenum(target), GLenum(pname), GLintArray(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)

glGetSeparableFilterEXT = platform.createExtensionFunction( 
'glGetSeparableFilterEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLenum,ctypes.c_void_p,ctypes.c_void_p,ctypes.c_void_p,),
doc='glGetSeparableFilterEXT(GLenum(target), GLenum(format), GLenum(type), c_void_p(row), c_void_p(column), c_void_p(span)) -> None',
argNames=('target','format','type','row','column','span',),
deprecated=_DEPRECATED,
)

glSeparableFilter2DEXT = platform.createExtensionFunction( 
'glSeparableFilter2DEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLsizei,constants.GLsizei,constants.GLenum,constants.GLenum,ctypes.c_void_p,ctypes.c_void_p,),
doc='glSeparableFilter2DEXT(GLenum(target), GLenum(internalformat), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), c_void_p(row), c_void_p(column)) -> None',
argNames=('target','internalformat','width','height','format','type','row','column',),
deprecated=_DEPRECATED,
)


def glInitConvolutionEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_perturb_normal

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_perturb_normal'
_DEPRECATED = False
GL_PERTURB_EXT = constant.Constant( 'GL_PERTURB_EXT', 0x85AE )
GL_TEXTURE_NORMAL_EXT = constant.Constant( 'GL_TEXTURE_NORMAL_EXT', 0x85AF )
glTextureNormalEXT = platform.createExtensionFunction( 
'glTextureNormalEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glTextureNormalEXT(GLenum(mode)) -> None',
argNames=('mode',),
deprecated=_DEPRECATED,
)


def glInitTexturePerturbNormalEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""String-array-handling code for PyOpenGL
"""
#from OpenGL.arrays._strings import dataPointer as old
from OpenGL import constants
from OpenGL.arrays import formathandler
import ctypes

psas = ctypes.pythonapi.PyString_AsString
# it's a c_char_p, but if we use that then the code will 
# attempt to use null-terminated versus arbitrarily sized
psas.restype = ctypes.c_size_t
#psas.restype = ctypes.c_char_p

def dataPointer( value, typeCode=None ):
    new = psas( ctypes.py_object(value) )
    return new

class StringHandler( formathandler.FormatHandler ):
    """String-specific data-type handler for OpenGL"""
    HANDLED_TYPES = (str, )
    @classmethod
    def from_param( cls, value, typeCode=None ):
        return ctypes.c_void_p( dataPointer( value ) )
    dataPointer = staticmethod( dataPointer )
    def zeros( self, dims, typeCode=None ):
        """Currently don't allow strings as output types!"""
        raise NotImplemented( """Don't currently support strings as output arrays""" )
    def ones( self, dims, typeCode=None ):
        """Currently don't allow strings as output types!"""
        raise NotImplemented( """Don't currently support strings as output arrays""" )
    def arrayToGLType( self, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        raise NotImplemented( """Can't guess data-type from a string-type argument""" )
    def arraySize( self, value, typeCode = None ):
        """Given a data-value, calculate ravelled size for the array"""
        # need to get bits-per-element...
        byteCount = BYTE_SIZES[ typeCode ]
        return len(value)//byteCount
    def arrayByteCount( self, value, typeCode = None ):
        """Given a data-value, calculate number of bytes required to represent"""
        return len(value)
    def asArray( self, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""
        if isinstance( value, str ):
            return value
        elif hasattr( value, 'tostring' ):
            return value.tostring()
        elif hasattr( value, 'raw' ):
            return value.raw
        # could convert types to string here, but we're not registered for 
        # anything save string types...
        raise TypeError( """String handler got non-string object: %r"""%(type(value)))
    def dimensions( self, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        raise TypeError(
            """Cannot calculate dimensions for a String data-type"""
        )

BYTE_SIZES = {
    constants.GL_DOUBLE: ctypes.sizeof( constants.GLdouble ),
    constants.GL_FLOAT: ctypes.sizeof( constants.GLfloat ),
    constants.GL_INT: ctypes.sizeof( constants.GLint ),
    constants.GL_SHORT: ctypes.sizeof( constants.GLshort ),
    constants.GL_UNSIGNED_BYTE: ctypes.sizeof( constants.GLubyte ),
    constants.GL_UNSIGNED_SHORT: ctypes.sizeof( constants.GLshort ),
    constants.GL_BYTE: ctypes.sizeof( constants.GLbyte ),
    constants.GL_UNSIGNED_INT: ctypes.sizeof( constants.GLuint ),
}


'''OpenGL extension NV.float_buffer

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_float_buffer'
_DEPRECATED = False
GL_FLOAT_R_NV = constant.Constant( 'GL_FLOAT_R_NV', 0x8880 )
GL_FLOAT_RG_NV = constant.Constant( 'GL_FLOAT_RG_NV', 0x8881 )
GL_FLOAT_RGB_NV = constant.Constant( 'GL_FLOAT_RGB_NV', 0x8882 )
GL_FLOAT_RGBA_NV = constant.Constant( 'GL_FLOAT_RGBA_NV', 0x8883 )
GL_FLOAT_R16_NV = constant.Constant( 'GL_FLOAT_R16_NV', 0x8884 )
GL_FLOAT_R32_NV = constant.Constant( 'GL_FLOAT_R32_NV', 0x8885 )
GL_FLOAT_RG16_NV = constant.Constant( 'GL_FLOAT_RG16_NV', 0x8886 )
GL_FLOAT_RG32_NV = constant.Constant( 'GL_FLOAT_RG32_NV', 0x8887 )
GL_FLOAT_RGB16_NV = constant.Constant( 'GL_FLOAT_RGB16_NV', 0x8888 )
GL_FLOAT_RGB32_NV = constant.Constant( 'GL_FLOAT_RGB32_NV', 0x8889 )
GL_FLOAT_RGBA16_NV = constant.Constant( 'GL_FLOAT_RGBA16_NV', 0x888A )
GL_FLOAT_RGBA32_NV = constant.Constant( 'GL_FLOAT_RGBA32_NV', 0x888B )
GL_TEXTURE_FLOAT_COMPONENTS_NV = constant.Constant( 'GL_TEXTURE_FLOAT_COMPONENTS_NV', 0x888C )
GL_FLOAT_CLEAR_COLOR_VALUE_NV = constant.Constant( 'GL_FLOAT_CLEAR_COLOR_VALUE_NV', 0x888D )
glget.addGLGetConstant( GL_FLOAT_CLEAR_COLOR_VALUE_NV, (4,) )
GL_FLOAT_RGBA_MODE_NV = constant.Constant( 'GL_FLOAT_RGBA_MODE_NV', 0x888E )
glget.addGLGetConstant( GL_FLOAT_RGBA_MODE_NV, (1,) )


def glInitFloatBufferNV():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension APPLE.element_array

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_element_array'
_DEPRECATED = False
GL_ELEMENT_ARRAY_APPLE = constant.Constant( 'GL_ELEMENT_ARRAY_APPLE', 0x8768 )
GL_ELEMENT_ARRAY_TYPE_APPLE = constant.Constant( 'GL_ELEMENT_ARRAY_TYPE_APPLE', 0x8769 )
glget.addGLGetConstant( GL_ELEMENT_ARRAY_TYPE_APPLE, (1,) )
GL_ELEMENT_ARRAY_POINTER_APPLE = constant.Constant( 'GL_ELEMENT_ARRAY_POINTER_APPLE', 0x876A )
glElementPointerAPPLE = platform.createExtensionFunction( 
'glElementPointerAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,ctypes.c_void_p,),
doc='glElementPointerAPPLE(GLenum(type), c_void_p(pointer)) -> None',
argNames=('type','pointer',),
deprecated=_DEPRECATED,
)

glDrawElementArrayAPPLE = platform.createExtensionFunction( 
'glDrawElementArrayAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLint,constants.GLsizei,),
doc='glDrawElementArrayAPPLE(GLenum(mode), GLint(first), GLsizei(count)) -> None',
argNames=('mode','first','count',),
deprecated=_DEPRECATED,
)

glDrawRangeElementArrayAPPLE = platform.createExtensionFunction( 
'glDrawRangeElementArrayAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,constants.GLint,constants.GLsizei,),
doc='glDrawRangeElementArrayAPPLE(GLenum(mode), GLuint(start), GLuint(end), GLint(first), GLsizei(count)) -> None',
argNames=('mode','start','end','first','count',),
deprecated=_DEPRECATED,
)

glMultiDrawElementArrayAPPLE = platform.createExtensionFunction( 
'glMultiDrawElementArrayAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,arrays.GLintArray,arrays.GLsizeiArray,constants.GLsizei,),
doc='glMultiDrawElementArrayAPPLE(GLenum(mode), GLintArray(first), GLsizeiArray(count), GLsizei(primcount)) -> None',
argNames=('mode','first','count','primcount',),
deprecated=_DEPRECATED,
)

glMultiDrawRangeElementArrayAPPLE = platform.createExtensionFunction( 
'glMultiDrawRangeElementArrayAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,arrays.GLintArray,arrays.GLsizeiArray,constants.GLsizei,),
doc='glMultiDrawRangeElementArrayAPPLE(GLenum(mode), GLuint(start), GLuint(end), GLintArray(first), GLsizeiArray(count), GLsizei(primcount)) -> None',
argNames=('mode','start','end','first','count','primcount',),
deprecated=_DEPRECATED,
)


def glInitElementArrayAPPLE():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.vertex_preclip

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_vertex_preclip'
_DEPRECATED = False
GL_VERTEX_PRECLIP_SGIX = constant.Constant( 'GL_VERTEX_PRECLIP_SGIX', 0x83EE )
GL_VERTEX_PRECLIP_HINT_SGIX = constant.Constant( 'GL_VERTEX_PRECLIP_HINT_SGIX', 0x83EF )


def glInitVertexPreclipSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""Implementation of GLU Nurbs structure and callback methods

Same basic pattern as seen with the gluTess* functions, just need to
add some bookkeeping to the structure class so that we can keep the
Python function references alive during the calling process.
"""
from OpenGL.raw import GLU as simple
from OpenGL import platform, converters, wrapper
from OpenGL.GLU import glustruct
from OpenGL.lazywrapper import lazy
from OpenGL import arrays, error
import ctypes
import weakref
from OpenGL.platform import PLATFORM
import OpenGL

__all__ = (
    'GLUnurbs',
    'gluNewNurbsRenderer',
    'gluNurbsCallback',
    'gluNurbsCallbackData',
    'gluNurbsCallbackDataEXT',
    'gluNurbsCurve',
    'gluNurbsSurface',
    'gluPwlCurve',
)

# /usr/include/GL/glu.h 242
class GLUnurbs(glustruct.GLUStruct, simple.GLUnurbs):
    """GLU Nurbs structure with oor and callback storage support
    
    IMPORTANT NOTE: the texture coordinate callback receives a raw ctypes 
    data-pointer, as without knowing what type of evaluation is being done 
    (1D or 2D) we cannot safely determine the size of the array to convert 
    it.  This is a limitation of the C implementation.  To convert to regular 
    data-pointer, just call yourNurb.ptrAsArray( ptr, size, arrays.GLfloatArray )
    with the size of data you expect.
    """
    FUNCTION_TYPE = PLATFORM.functionTypeFor(PLATFORM.GLU)
    CALLBACK_FUNCTION_REGISTRARS = {
        # mapping from "which" to a function that should take 3 parameters,
        # the nurb, the which and the function pointer...
    }
    CALLBACK_TYPES = {
        # mapping from "which" GLU enumeration to a ctypes function type
        simple.GLU_NURBS_BEGIN: FUNCTION_TYPE( 
            None, simple.GLenum 
        ),
        simple.GLU_NURBS_BEGIN_DATA: FUNCTION_TYPE( 
            None, simple.GLenum, ctypes.POINTER(simple.GLvoid) 
        ),
        simple.GLU_NURBS_VERTEX: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat)
        ),
        simple.GLU_NURBS_VERTEX_DATA: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
        ),
        simple.GLU_NURBS_NORMAL: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat)
        ),
        simple.GLU_NURBS_NORMAL_DATA: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
        ),
        simple.GLU_NURBS_COLOR: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat)
        ),
        simple.GLU_NURBS_COLOR_DATA: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
        ),
        simple.GLU_NURBS_TEXTURE_COORD: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat)
        ),
        simple.GLU_NURBS_TEXTURE_COORD_DATA: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLfloat), ctypes.POINTER(simple.GLvoid) 
        ),
        simple.GLU_NURBS_END:FUNCTION_TYPE( 
            None
        ),
        simple.GLU_NURBS_END_DATA: FUNCTION_TYPE( 
            None, ctypes.POINTER(simple.GLvoid) 
        ),
        simple.GLU_NURBS_ERROR:FUNCTION_TYPE( 
            None, simple.GLenum, 
        ),
    }
    WRAPPER_METHODS = {
        simple.GLU_NURBS_BEGIN: None,
        simple.GLU_NURBS_BEGIN_DATA: '_justOOR',
        simple.GLU_NURBS_VERTEX: '_vec3',
        simple.GLU_NURBS_VERTEX_DATA: '_vec3',
        simple.GLU_NURBS_NORMAL: '_vec3',
        simple.GLU_NURBS_NORMAL_DATA: '_vec3',
        simple.GLU_NURBS_COLOR: '_vec4',
        simple.GLU_NURBS_COLOR_DATA: '_vec4',
        simple.GLU_NURBS_TEXTURE_COORD: '_tex',
        simple.GLU_NURBS_TEXTURE_COORD_DATA: '_tex',
        simple.GLU_NURBS_END: None,
        simple.GLU_NURBS_END_DATA: '_justOOR',
        simple.GLU_NURBS_ERROR: None,
    }
    def _justOOR( self, function ):
        """Just do OOR on the last argument..."""
        def getOOR( *args ):
            args = args[:-1] + (self.originalObject(args[-1]),)
            return function( *args )
        return getOOR
    def _vec3( self, function, size=3 ):
        """Convert first arg to size-element array, do OOR on arg2 if present"""
        def vec( *args ):
            vec = self.ptrAsArray(args[0],size,arrays.GLfloatArray)
            if len(args) > 1:
                oor = self.originalObject(args[1])
                return function( vec, oor )
            else:
                return function( vec )
        return vec
    def _vec4( self, function ):
        """Size-4 vector version..."""
        return self._vec3( function, 4 )
    def _tex( self, function ):
        """Texture coordinate callback 
        
        NOTE: there is no way for *us* to tell what size the array is, you will 
        get back a raw data-point, not an array, as you do for all other callback 
        types!!!
        """
        def oor( *args ):
            if len(args) > 1:
                oor = self.originalObject(args[1])
                return function( args[0], oor )
            else:
                return function( args[0] )
        return oor

# XXX yes, this is a side-effect...
simple.gluNewNurbsRenderer.restype = ctypes.POINTER( GLUnurbs )

def _callbackWithType( funcType ):
    """Get gluNurbsCallback function with set last arg-type"""
    result =  platform.copyBaseFunction(
        simple.gluNurbsCallback
    )
    result.argtypes = [ctypes.POINTER(GLUnurbs), simple.GLenum, funcType]
    assert result.argtypes[-1] == funcType
    return result

for (c,funcType) in GLUnurbs.CALLBACK_TYPES.items():
    cb = _callbackWithType( funcType )
    GLUnurbs.CALLBACK_FUNCTION_REGISTRARS[ c ] = cb
    assert funcType == GLUnurbs.CALLBACK_TYPES[c]
    assert cb.argtypes[-1] == funcType
del c,cb, funcType

def gluNurbsCallback( nurb, which, CallBackFunc ):
    """Dispatch to the nurb's addCallback operation"""
    return nurb.addCallback( which, CallBackFunc )

@lazy( simple.gluNewNurbsRenderer )
def gluNewNurbsRenderer( baseFunction ):
    """Return a new nurbs renderer for the system (dereferences pointer)"""
    newSet = baseFunction()
    new = newSet[0]
    #new.__class__ = GLUnurbs # yes, I know, ick
    return new

@lazy( simple.gluNurbsCallbackData )
def gluNurbsCallbackData( baseFunction, nurb, userData ):
    """Note the Python object for use as userData by the nurb"""
    return baseFunction( 
        nurb, nurb.noteObject( userData ) 
    )

MAX_ORDER = 8
def checkOrder( order,knotCount,name ):
    """Check that order is valid..."""
    if order < 1:
        raise error.GLUError( 
            """%s should be 1 or more, is %s"""%( name,order,) 
        )
    elif order > MAX_ORDER:
        raise error.GLUError( 
            """%s should be %s or less, is %s"""%( name, MAX_ORDER, order) 
        )
    elif knotCount < (2*order):
        raise error.GLUError( 
            """Knotcount must be at least 2x %s is %s should be at least %s"""%( name, knotCount, 2*order) 
        )
def checkKnots( knots, name ):
    """Check that knots are in ascending order"""
    if len(knots):
        knot = knots[0]
        for next in knots[1:]:
            if next < knot:
                raise error.GLUError(
                    """%s has decreasing knot %s after %s"""%( name, next, knot )
                )

@lazy( simple.gluNurbsCallbackDataEXT )
def gluNurbsCallbackDataEXT( baseFunction,nurb, userData ):
    """Note the Python object for use as userData by the nurb"""
    return baseFunction( 
        nurb, nurb.noteObject( userData ) 
    )

@lazy( simple.gluNurbsCurve )
def gluNurbsCurve( baseFunction, nurb, knots, control, type ):
    """Pythonic version of gluNurbsCurve
    
    Calculates knotCount, stride, and order automatically
    """
    knots = arrays.GLfloatArray.asArray( knots )
    knotCount = arrays.GLfloatArray.arraySize( knots )
    control = arrays.GLfloatArray.asArray( control )
    try:
        length,step = arrays.GLfloatArray.dimensions( control )
    except ValueError, err:
        raise error.GLUError( """Need a 2-dimensional control array""" )
    order = knotCount - length
    if OpenGL.ERROR_CHECKING:
        checkOrder( order, knotCount, 'order of NURBS curve')
        checkKnots( knots, 'knots of NURBS curve')
    return baseFunction(
        nurb, knotCount, knots, step, control, order, type,
    )

@lazy( simple.gluNurbsSurface )
def gluNurbsSurface( baseFunction, nurb, sKnots, tKnots, control, type ):
    """Pythonic version of gluNurbsSurface
    
    Calculates knotCount, stride, and order automatically
    """
    sKnots = arrays.GLfloatArray.asArray( sKnots )
    sKnotCount = arrays.GLfloatArray.arraySize( sKnots )
    tKnots = arrays.GLfloatArray.asArray( tKnots )
    tKnotCount = arrays.GLfloatArray.arraySize( tKnots )
    control = arrays.GLfloatArray.asArray( control )

    try:
        length,width,step = arrays.GLfloatArray.dimensions( control )
    except ValueError, err:
        raise error.GLUError( """Need a 3-dimensional control array""" )
    sOrder = sKnotCount - length 
    tOrder = tKnotCount - width 
    sStride = width*step
    tStride = step
    if OpenGL.ERROR_CHECKING:
        checkOrder( sOrder, sKnotCount, 'sOrder of NURBS surface')
        checkOrder( tOrder, tKnotCount, 'tOrder of NURBS surface')
        checkKnots( sKnots, 'sKnots of NURBS surface')
        checkKnots( tKnots, 'tKnots of NURBS surface')
    if not (sKnotCount-sOrder)*(tKnotCount-tOrder) == length*width:
        raise error.GLUError(
            """Invalid NURB structure""",
            nurb, sKnotCount, sKnots, tKnotCount, tKnots,
            sStride, tStride, control,
            sOrder,tOrder,
            type
        )

    result = baseFunction(
        nurb, sKnotCount, sKnots, tKnotCount, tKnots,
        sStride, tStride, control,
        sOrder,tOrder,
        type
    )
    return result

@lazy( simple.gluPwlCurve )
def gluPwlCurve( baseFunction, nurb, data, type ):
    """gluPwlCurve -- piece-wise linear curve within GLU context
    
    data -- the data-array 
    type -- determines number of elements/data-point
    """
    data = arrays.GLfloatArray.asArray( data )
    if type == simple.GLU_MAP1_TRIM_2:
        divisor = 2
    elif type == simple.GLU_MAP_TRIM_3:
        divisor = 3
    else:
        raise ValueError( """Unrecognised type constant: %s"""%(type))
    size = arrays.GLfloatArray.arraySize( data )
    size = int(size//divisor)
    return baseFunction( nurb, size, data, divisor, type )



'''OpenGL extension SGIX.framezoom

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_framezoom'
_DEPRECATED = False
GL_FRAMEZOOM_SGIX = constant.Constant( 'GL_FRAMEZOOM_SGIX', 0x818B )
GL_FRAMEZOOM_FACTOR_SGIX = constant.Constant( 'GL_FRAMEZOOM_FACTOR_SGIX', 0x818C )
GL_MAX_FRAMEZOOM_FACTOR_SGIX = constant.Constant( 'GL_MAX_FRAMEZOOM_FACTOR_SGIX', 0x818D )
glFrameZoomSGIX = platform.createExtensionFunction( 
'glFrameZoomSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,),
doc='glFrameZoomSGIX(GLint(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)


def glInitFramezoomSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.coordinate_frame

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.coordinate_frame to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows specifying a per-vertex tangent and binormal
	vector in addition to the normal vector, defining a coordinate frame.
	The coordinate frame is used in additional extensions which also build
	on fragment lighting to achieve bump mapping.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/coordinate_frame.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.coordinate_frame import *
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.transform_feedback

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_transform_feedback'
_DEPRECATED = False
GL_TRANSFORM_FEEDBACK_BUFFER_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_EXT', 0x8C8E )
GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT', 0x8C84 )
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT', 0x8C85 )
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT', 0x8C8F )
GL_INTERLEAVED_ATTRIBS_EXT = constant.Constant( 'GL_INTERLEAVED_ATTRIBS_EXT', 0x8C8C )
GL_SEPARATE_ATTRIBS_EXT = constant.Constant( 'GL_SEPARATE_ATTRIBS_EXT', 0x8C8D )
GL_PRIMITIVES_GENERATED_EXT = constant.Constant( 'GL_PRIMITIVES_GENERATED_EXT', 0x8C87 )
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT', 0x8C88 )
GL_RASTERIZER_DISCARD_EXT = constant.Constant( 'GL_RASTERIZER_DISCARD_EXT', 0x8C89 )
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT', 0x8C8A )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT', 0x8C8B )
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT = constant.Constant( 'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT', 0x8C80 )
GL_TRANSFORM_FEEDBACK_VARYINGS_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYINGS_EXT', 0x8C83 )
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT', 0x8C7F )
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT = constant.Constant( 'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT', 0x8C76 )
glBeginTransformFeedbackEXT = platform.createExtensionFunction( 
'glBeginTransformFeedbackEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glBeginTransformFeedbackEXT(GLenum(primitiveMode)) -> None',
argNames=('primitiveMode',),
deprecated=_DEPRECATED,
)

glEndTransformFeedbackEXT = platform.createExtensionFunction( 
'glEndTransformFeedbackEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(),
doc='glEndTransformFeedbackEXT() -> None',
argNames=(),
deprecated=_DEPRECATED,
)

glBindBufferRangeEXT = platform.createExtensionFunction( 
'glBindBufferRangeEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,constants.GLintptr,constants.GLsizeiptr,),
doc='glBindBufferRangeEXT(GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset), GLsizeiptr(size)) -> None',
argNames=('target','index','buffer','offset','size',),
deprecated=_DEPRECATED,
)

glBindBufferOffsetEXT = platform.createExtensionFunction( 
'glBindBufferOffsetEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,constants.GLintptr,),
doc='glBindBufferOffsetEXT(GLenum(target), GLuint(index), GLuint(buffer), GLintptr(offset)) -> None',
argNames=('target','index','buffer','offset',),
deprecated=_DEPRECATED,
)

glBindBufferBaseEXT = platform.createExtensionFunction( 
'glBindBufferBaseEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLuint,),
doc='glBindBufferBaseEXT(GLenum(target), GLuint(index), GLuint(buffer)) -> None',
argNames=('target','index','buffer',),
deprecated=_DEPRECATED,
)

glTransformFeedbackVaryingsEXT = platform.createExtensionFunction( 
'glTransformFeedbackVaryingsEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,ctypes.POINTER( ctypes.POINTER( constants.GLchar )),constants.GLenum,),
doc='glTransformFeedbackVaryingsEXT(GLuint(program), GLsizei(count), POINTER( ctypes.POINTER( constants.GLchar ))(varyings), GLenum(bufferMode)) -> None',
argNames=('program','count','varyings','bufferMode',),
deprecated=_DEPRECATED,
)

glGetTransformFeedbackVaryingEXT = platform.createExtensionFunction( 
'glGetTransformFeedbackVaryingEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLsizeiArray,arrays.GLuintArray,arrays.GLcharArray,),
doc='glGetTransformFeedbackVaryingEXT(GLuint(program), GLuint(index), GLsizei(bufSize), GLsizeiArray(length), GLsizeiArray(size), GLuintArray(type), GLcharArray(name)) -> None',
argNames=('program','index','bufSize','length','size','type','name',),
deprecated=_DEPRECATED,
)


def glInitTransformFeedbackEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.resample

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_resample'
_DEPRECATED = False
GL_PACK_RESAMPLE_SGIX = constant.Constant( 'GL_PACK_RESAMPLE_SGIX', 0x842C )
glget.addGLGetConstant( GL_PACK_RESAMPLE_SGIX, (1,) )
GL_UNPACK_RESAMPLE_SGIX = constant.Constant( 'GL_UNPACK_RESAMPLE_SGIX', 0x842D )
glget.addGLGetConstant( GL_UNPACK_RESAMPLE_SGIX, (1,) )
GL_RESAMPLE_REPLICATE_SGIX = constant.Constant( 'GL_RESAMPLE_REPLICATE_SGIX', 0x842E )
GL_RESAMPLE_ZERO_FILL_SGIX = constant.Constant( 'GL_RESAMPLE_ZERO_FILL_SGIX', 0x842F )
GL_RESAMPLE_DECIMATE_SGIX = constant.Constant( 'GL_RESAMPLE_DECIMATE_SGIX', 0x8430 )


def glInitResampleSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )
